/*
 Standalone Temperature Logger for ATtiny85
 Version: 1.0
 Author: electricteardown (http://www.insidegadgets.com)
 Created: 9/01/2011
 Last Modified: 22/01/2011
 
 Used to monitor temperature every x minutes/hours which is saved 
 to the built in EEPROM. 
 
 */

#include <math.h>
#include <EEPROM.h>

int ledPin = 0;
int thermistorPin = 1; // Analog input 1
int buttonPin = 4;
int dataPin = 1;
int clockPin = 3;

int functionSelect = 0;
int dataCount = 1;
int delayTime = -1;

void setup() {
  pinMode(ledPin, OUTPUT);
  pinMode(buttonPin, INPUT);  
  pinMode(clockPin, INPUT);
  pinMode(dataPin, OUTPUT);
}

void loop() {
  
  // Wait for input
  if (functionSelect == 0) {
    
    // Read delay time
    if (delayTime == -1) {
      delayTime = EEPROM.read(0);
      if (delayTime > 10) { // Not a normal value
        delayTime = 0;
      }
    }
    
    // Always reset variables in case
    digitalWrite(ledPin, LOW);
    digitalWrite(dataPin, LOW);
    
    // Check button press, if still pressed after 500ms, check again
    // after 1500ms, if still pressed go to configuration
    int buttonState = digitalRead(buttonPin);
    if (buttonState == HIGH) {
      delay(500);
      buttonState = digitalRead(buttonPin);
      if (buttonState == true) {
        delay(1500);
        buttonState = digitalRead(buttonPin);
        if (buttonState == HIGH) {
          functionSelect = 2;
          delayTime = -1;
          digitalWrite(ledPin, HIGH);
          delay(2000);
          digitalWrite(ledPin, LOW);
        }
      }
      else {
        functionSelect = 1;
        for (int x = 0; x < 3; x++) {
          digitalWrite(ledPin, HIGH);
          delay(500);
          digitalWrite(ledPin, LOW);
          delay(500);
        }
      }
    }
    
    int clockState = digitalRead(clockPin);
    if (clockState == HIGH) {
      int clockhighCount = 0;
      // Delay for 10 x 25ms = 250ms, check CLK is always high
      for (int x = 0; x < 10; x++) { 
        delay(25);
        clockState = digitalRead(clockPin);
        if (clockState == HIGH) {
          clockhighCount++;
        }
        else {
          clockhighCount = 0;
        }
      }
      // CLK is high, now give the 250ms HIGH as an ok
      if (clockhighCount >= 10) {
        functionSelect = 3;
        digitalWrite(dataPin, HIGH);
        delay(250);
        digitalWrite(dataPin, LOW);
      }
    }
  }
  
  // Log Temperature
  if (functionSelect == 1) {
    int tempValue = int(Thermistor(analogRead(thermistorPin)));
    
    // Calculations to store -40C to 125C int to a single byte
    byte tempStore = tempValue + 130;
    if (tempValue <= -41) { // Temp is over min temp accepted
      tempStore = 90;
    }
    if (tempValue >= 126) { // Temp is over max temp accepted
      tempStore = 255;
    }
    
    // Write to EEPROM
    if (dataCount < 512) {
      EEPROM.write(dataCount, tempStore);
      dataCount++;
      if (dataCount < 512) {
        EEPROM.write(dataCount, 0); // So we know where to stop when extracting data
      }
    }
    else {
      functionSelect = 0;
    }
        
    // Calculate the delay time
    if (delayTime == 0) {
      delay(60000);
    }
    if (delayTime >= 1 && delayTime < 4) {
      delay((delayTime * 5) * 60000);
    }
    if (delayTime >= 4 && delayTime < 6) {
      delay(((delayTime-3) * 30) * 60000);
    }
    if (delayTime >= 6 && delayTime < 8) {
      delay(((delayTime-5) * 120) * 60000);
    }
    if (delayTime == 8) {
      delay(8 * 60 * 60000);
    }
    if (delayTime == 9) {
      delay(12 * 60 * 60000);
    }
    if (delayTime == 10) {
      delay(24 * 60 * 60000);
    }
  }
  
  // Configure the delay time
  if (functionSelect == 2) {
    int buttonState = digitalRead(buttonPin);
    if (buttonState == HIGH) {
      delay(500);
      int buttonPressed = false;
      buttonState = digitalRead(buttonPin);
      if (buttonState == HIGH && delayTime >= 0) {
        delay(1500);
        buttonState = digitalRead(buttonPin);
        if (buttonState == HIGH) {
          functionSelect = 0;
          EEPROM.write(0, delayTime);
          for (int x = 0; x < 3; x++) {
            digitalWrite(ledPin, HIGH);
            delay(500);
            digitalWrite(ledPin, LOW);
            delay(500);
          }
        }
        else {
          buttonPressed = true;
        }
      }
      if (buttonState == LOW || buttonPressed == true) {
        delayTime++;
        digitalWrite(ledPin, HIGH);
        delay(500);
        digitalWrite(ledPin, LOW);
        
        if (delayTime > 10) { // Too many presses
          delayTime = -1;
          digitalWrite(ledPin, HIGH);
          delay(1500);
          digitalWrite(ledPin, LOW);
        }
      }
    }
  }
  
  // Send data from EEPROM to Arduino
  if (functionSelect == 3) {
    digitalWrite(ledPin, HIGH);
    boolean inTransfer = true;
    int readCount = 1;
    while (inTransfer == true) {
      byte dataByte = 0;
      if (readCount < 512) {
        dataByte = EEPROM.read(readCount);
      }
      for (int x = 7; x >= 0; x--) {
        digitalWrite(dataPin, LOW);
        
        // Wait until CLK is low
        long resetTimer = millis();
        int clockIn = digitalRead(clockPin);
        while (clockIn == HIGH && inTransfer == true) {
          clockIn = digitalRead(clockPin);
          if (millis() > resetTimer + 250) {  // Check if host is restarting the CLK
            inTransfer = false;
            functionSelect = 0;
          }          
        }
        
        // Output the bit
        boolean dataBit = bitRead(dataByte, x);
        digitalWrite(dataPin, dataBit);
        
        // Wait until CLK is high
        resetTimer = millis();
        clockIn = digitalRead(clockPin);
        while (clockIn == LOW && inTransfer == true) {
          clockIn = digitalRead(clockPin);
          if (millis() > resetTimer + 100) {
            digitalWrite(dataPin, LOW); // Don't want to wait 250ms for this otherwise Arduino will think it's beginning from the start again
          }
          if (millis() > resetTimer + 250) { // Check if host is restarting the CLK
            inTransfer = false;
            functionSelect = 0;
          }
        }
      }
      if (dataByte == 0) { // No more data to send
        inTransfer = false;
        functionSelect = 0;
      }
      readCount++;
    }
  }
}

// Taken from http://www.arduino.cc/playground/ComponentLib/Thermistor2
double Thermistor(int RawADC) {
  double Temp;
  Temp = log(((10240000/RawADC) - 10000));
  Temp = 1 / (0.001129148 + (0.000234125 * Temp) + (0.0000000876741 * Temp * Temp * Temp));
  Temp = Temp - 273.15;            // Convert Kelvin to Celcius
  return Temp;
}

